<?php

namespace Drupal\pluginreference\Controller;

use Drupal\Component\Utility\SortArray;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller class for the pluginreference autocomplete form element.
 *
 * @package Drupal\pluginreference\Controller
 */
class PluginReferenceAutocompleteController extends ControllerBase {

  /**
   * The Drupal container.
   *
   * @var \Symfony\Component\DependencyInjection\ContainerInterface
   */
  protected $container;

  /**
   * Constructs a EntityAutocompleteController object.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Drupal container.
   */
  public function __construct(ContainerInterface $container) {
    $this->container = $container;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container);
  }

  /**
   * Autocomplete callback.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   * @param string $target_type
   *   The pluginreference type to filter results.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON autocomplete response.
   */
  public function autocomplete(Request $request, $target_type) {
    $matches = [];
    if ($this->container->get('current_user')->hasPermission('pluginreference autocomplete view results') &&
      ($input = $request->query->get('q')) && $this->container->has('plugin.manager.' . $target_type)) {
      foreach ($this->container->get('plugin.manager.' . $target_type)->getDefinitions() as $plugin_type_id => $plugin_definition) {
        $label = $plugin_definition['label'] ?? $plugin_definition['admin_label'] ?? $plugin_type_id;
        if (stripos($label, $input) !== FALSE) {
          $matches[] = [
            'value' => $label . ' (' . $plugin_type_id . ')',
            'label' => $label,
          ];
        }
      }
      usort($matches, [$this, 'sortByLabelElement']);
    }

    return new JsonResponse($matches);
  }

  /**
   * Sorts a structured array by 'label' element.
   *
   * Callback for uasort().
   *
   * @param array $a
   *   First item for comparison. The compared items should be associative
   *   arrays hat optionally include a 'label' element.
   * @param array $b
   *   Second item for comparison.
   *
   * @return int
   *   The comparison result for uasort().
   */
  protected static function sortByLabelElement(array $a, array $b) {
    return SortArray::sortByKeyString($a, $b, 'label');
  }

}
