<?php

namespace Drupal\pluginreference\Plugin\Field\FieldType;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\MapDataDefinition;

/**
 * Plugin implementation of the 'plugin_reference' entity field type.
 *
 * @FieldType(
 *   id = "plugin_reference",
 *   label = @Translation("Plugin"),
 *   description = @Translation("This field stores a soft reference to a plugin"),
 *   category = @Translation("Reference"),
 *   default_widget = "plugin_reference_select",
 *   default_formatter = "plugin_reference_id",
 * )
 */
class PluginReferenceItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    return [
      'target_type' => '',
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function mainPropertyName() {
    return 'plugin_id';
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    // Prevent early t() calls by using the TranslatableMarkup.
    $properties['plugin_id'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Text value'))
      ->addConstraint('Length', ['max' => 255])
      ->setRequired(TRUE);

    $properties['configuration'] = MapDataDefinition::create()
      ->setLabel(t('Configuration'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    $schema = [
      'columns' => [
        'plugin_id' => [
          'type' => 'varchar',
          'length' => 255,
        ],
        'configuration' => [
          'description' => 'Serialized array of plugin configuration.',
          'type' => 'blob',
          'size' => 'big',
          'serialize' => TRUE,
        ],
      ],
      'indexes' => ['plugin_id' => ['plugin_id']],
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    $values = NULL;
    if (\Drupal::getContainer()->has('plugin.manager.' . $field_definition->getSetting('target_type'))) {
      $definitions = \Drupal::getContainer()
        ->get('plugin.manager.' . $field_definition->getSetting('target_type'))
        ->getDefinitions();
      shuffle($definitions);
      $values['plugin_id'] = key($definitions);
      $values['configuration'] = [];
    }
    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    $elements = [];
    $options = [];

    $container = \Drupal::getContainer();
    foreach ($container->getServiceIds() as $serviceId) {
      if (strpos($serviceId, 'plugin.manager.') === 0) {
        $service = $container->get($serviceId);
        $typeName = substr($serviceId, 15);
        $class = get_class($service);
        $class_parts = preg_split('/\\\/', $class);
        $provider = $class_parts[1];
        if ($provider === 'Core') {
          $provider = 'system';
        }
        $options[\Drupal::moduleHandler()->getName($provider)][$typeName] = $class;
      }
    }

    $elements['target_type'] = [
      '#type' => 'select',
      '#title' => new TranslatableMarkup('Type of plugin to reference'),
      '#options' => $options,
      '#default_value' => $this->getSetting('target_type'),
      '#required' => TRUE,
      '#disabled' => $has_data,
      '#size' => 1,
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $plugin_id = $this->get('plugin_id')->getValue();
    return $plugin_id === NULL || $plugin_id === '';
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    // Treat the values as property value of the main property, if no array is
    // given.
    if (isset($values) && !is_array($values)) {
      $values = [static::mainPropertyName() => $values];
    }
    if (isset($values)) {
      $values += [
        'configuration' => [],
      ];
    }
    // Unserialize the values.
    if (is_string($values['configuration'])) {
      $values['configuration'] = unserialize($values['configuration']);
    }
    parent::setValue($values, $notify);
  }

  /**
   * Get the initialized plugin.
   *
   * @return \Drupal\Component\Plugin\PluginBase|null
   *   The initialized plugin.
   */
  public function getReferencedPlugin(): ?PluginBase {
    if (empty($this->plugin_id)) {
      return NULL;
    }

    $plugin_manager = $this->getPluginManager();
    return $plugin_manager->createInstance($this->plugin_id, $this->configuration ?? []);

  }

  /**
   * Helper method that returns the plugin manager.
   *
   * @return \Drupal\Component\Plugin\PluginManagerInterface
   *   The plugin manager.
   */
  protected function getPluginManager(): PluginManagerInterface {
    return \Drupal::getContainer()
      ->get('plugin.manager.' . $this->getFieldDefinition()->getSetting('target_type'));
  }

}
