<?php

namespace Drupal\pluginreference\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;

/**
 * Trait that handles the configuration form on plugin reference field widgets.
 */
trait PluginReferenceConfigurationFormTrait {

  /**
   * Defines the configuration settings..
   *
   * @return string[]
   *   A list of the configuration settings, keyed by the setting name.
   */
  public static function configurationDefaultSettings() {
    return [
      'configuration_form' => 'full',
    ];
  }

  /**
   * The configuration settings form.
   *
   * @param array $form
   *   The form where the configuration settings form is being included in.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form definition for the configugration part.
   */
  public function configurationSettingsForm(array $form, FormStateInterface $form_state) {
    $elements['configuration_form'] = [
      '#type' => 'select',
      '#title' => $this->t('Configuration form'),
      '#description' => $this->t('How the plugin configuration form will be shown.'),
      '#options' => [
        'full' => $this->t('Full'),
        'hidden' => $this->t('Hidden'),
      ],
      '#default_value' => $this->getSetting('configuration_form'),
      '#required' => TRUE,
    ];

    return $elements;
  }

  /**
   * Returns a short summary for the configuration settings.
   *
   * @return array
   *   A short summary of the configuration settings.
   */
  public function configurationSettingsSummary() {
    $summary[] = $this->t('Configuration form: @configuration_form', ['@configuration_form' => $this->getSetting('configuration_form')]);
    return $summary;
  }

  /**
   * The configuration form element.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   Array of default values for this field.
   * @param int $delta
   *   The order of this item in the array of sub-elements (0, 1, 2, etc.).
   * @param array $element
   *   A form element array containing basic properties for the widget.
   * @param array $form
   *   The form structure where widgets are being attached to. This might be a
   *   full form structure, or a sub-element of a larger form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form elements for the configuration part.
   */
  public function configurationFormElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\pluginreference\Plugin\Field\FieldType\PluginReferenceItem $item */
    $item =& $items[$delta];

    $field_name = $this->fieldDefinition->getName();
    $configuration_id = implode('-', array_merge(
      $element['#field_parents'],
      [$field_name, $delta, 'configuration']
    ));

    $values = $form_state->getValues();
    $item->plugin_id = (isset($values[$field_name][$delta]['plugin_id'])) ? $values[$field_name][$delta]['plugin_id'] : $item->plugin_id;
    if (!empty($values[$field_name][$delta]['configuration'])) {
      $item->configuration = $values[$field_name][$delta]['configuration'];
    }
    else {
      $item->configuration = $item->configuration ?: [];
    }

    $element['#target_type'] = $this->fieldDefinition->getSetting('target_type');

    // Show configuration form if required.
    if ($this->getSetting('configuration_form') === 'full') {
      $element['plugin_id']['#ajax'] = [
        'callback' => [$this, 'configurationForm'],
        'wrapper' => $configuration_id,
      ];

      if ($this->pluginId === 'plugin_reference_autocomplete') {
        $element['plugin_id']['#ajax']['event'] = 'autocompleteclose';
        $element['plugin_id']['#ajax']['progress'] = FALSE;
      }

      // Build configuration container.
      $element['configuration'] = [
        '#type' => 'container',
        '#attributes' => ['id' => $configuration_id],
        '#tree' => TRUE,
      ];

      // If the plugin exists get the plugin's configuration form.
      if ($item instanceof FieldItemInterface && $plugin_instance = $item->getReferencedPlugin()) {
        // Some blocks like 'system_branding_block rely on block_theme that is
        // set in the form_state object.
        // @see \Drupal\system\Plugin\Block\SystemBrandingBlock::blockForm().
        $form_state->set('block_theme', \Drupal::config('system.theme')->get('default'));
        // Create a separate variable for the plugin configuration form, so
        // plugins expecting the form array by reference don't throw an error.
        $plugin_configuration_form = [];
        $element['configuration'] += $plugin_instance->buildConfigurationForm($plugin_configuration_form, $form_state);
        $element['#element_validate'] = [[$this, 'configurationValidate']];
      }
    }

    return $element;
  }

  /**
   * The configuration validation checks.
   *
   * @param array $element
   *   A form element array containing basic properties for the widget.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param array $form
   *   The form where the configuration settings form is being included in.
   */
  public function configurationValidate(array $element, FormStateInterface $form_state, array $form) {
    $plugin_manager = \Drupal::service('plugin.manager.' . $element['#target_type']);

    $values = $form_state->getValues();
    $plugin_id = NestedArray::getValue($values, $element['plugin_id']['#parents']);

    if (!empty($plugin_id) && $plugin_manager->hasDefinition($plugin_id)) {
      // Clean up configuration settings.
      $configuration = NestedArray::getValue($values, $element['configuration']['#parents']) ?? [];

      // Execute the plugin validate configuration.
      $plugin_instance = $plugin_manager->createInstance($plugin_id, $configuration);
      $configuration = (new FormState())->setValues($configuration);
      $plugin_instance->validateConfigurationForm($element['configuration'], $configuration);

      // Pass along errors from the plugin validation.
      foreach ($configuration->getErrors() as $key => $error) {
        $parents = implode('][', $element['configuration']['#parents']);
        // If the plugin form used setError() then the parents will already be
        // part of the key since we are passing along the element in the context
        // of the whole form. If the plugin form used setErrorByName we need to
        // add the parents in.
        if (strpos($key, $parents) === FALSE) {
          $key = sprintf('%s][%s', $parents, $key);
        }
        $form_state->setErrorByName($key, $error);
      }

      // Call submitConfiguration, since it's possible that the values are
      // altered in the submitConfigurationForm method.
      $plugin_instance->submitConfigurationForm($element['configuration'], $configuration);
      NestedArray::setValue($values, $element['configuration']['#parents'], $plugin_instance->getConfiguration());
      $form_state->setValues($values);
    }
    else {
      // Clear all configuration settings.
      NestedArray::setValue($values, $element['configuration']['#parents'], []);
    }
  }

  /**
   * Ajax callback that return plugin configuration form.
   *
   * @param array $form
   *   The form where the configuration settings form is being included in.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return mixed
   *   The configuration form element.
   */
  public function configurationForm(array $form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    $array_parents = $trigger_element['#array_parents'];
    $array_parents[(count($array_parents) - 1)] = 'configuration';
    return NestedArray::getValue($form, $array_parents);
  }

}
