<?php

namespace Drupal\pluginreference\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'plugin_reference_select' widget.
 *
 * @FieldWidget(
 *   id = "plugin_reference_select",
 *   label = @Translation("Select list"),
 *   field_types = {
 *     "plugin_reference"
 *   }
 * )
 */
class PluginReferenceSelectWidget extends WidgetBase {

  use PluginReferenceConfigurationFormTrait;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The Plugin Manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $pluginManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    /** @var \Drupal\field\Entity\FieldConfig $field_definition */
    $field_definition = $configuration['field_definition'];
    $target_type = $field_definition->getFieldStorageDefinition()
      ->getSetting('target_type');

    if ($container->has('plugin.manager.' . $target_type)) {
      $instance->pluginManager = $container->get('plugin.manager.' . $target_type);
    }

    $instance->moduleHandler = $container->get('module_handler');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'provider_grouping' => TRUE,
    ] + self::configurationDefaultSettings() + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $elements['provider_grouping'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Group per provider'),
      '#description' => $this->t('Group the options per provider.'),
      '#default_value' => $this->getSetting('provider_grouping'),
    ];
    return $elements + $this->configurationSettingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    return $summary + $this->configurationSettingsSummary();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element['plugin_id'] = [
      '#type' => 'select',
      '#default_value' => $items[$delta]->plugin_id ?? NULL,
      '#options' => $this->getOptions($element['#required']),
    ] + $element;

    $element = $this->configurationFormElement($items, $delta, $element, $form, $form_state);

    return $element;
  }

  /**
   * Build the options used by this widget.
   *
   * @param bool $required
   *   Is the field required.
   *
   * @return array
   *   An array of options.
   */
  protected function getOptions(bool $required): array {
    $options = [];

    if ((bool) $this->getSetting('provider_grouping') === TRUE) {
      foreach ($this->pluginManager->getDefinitions() as $plugin_type_id => $plugin_definition) {
        $provider = $plugin_definition['provider'] === 'core' ? 'system' : $plugin_definition['provider'];
        $options[$this->moduleHandler->getName($provider)][$plugin_type_id] = $plugin_definition['label'] ?? $plugin_definition['admin_label'] ?? $plugin_type_id;
      }

      // Since the items are grouped per module, first sort on the groups.
      ksort($options);
      // Then sort all options in every group.
      foreach ($options as $group => $plugins) {
        natcasesort($options[$group]);
      }
    }
    else {
      foreach ($this->pluginManager->getDefinitions() as $plugin_type_id => $plugin_definition) {
        $options[$plugin_type_id] = $plugin_definition['label'] ?? $plugin_definition['admin_label'] ?? $plugin_type_id;
      }

      // Sort all items alphabetically.
      natcasesort($options);
    }

    if (!$required) {
      $options = ['' => $this->t('- None -')] + $options;
    }

    return $options;
  }

}
